package db

import (
	"gorm.io/driver/sqlite"
	"gorm.io/gorm"
	// "log"
	"math/rand"
	"time"
)

type User struct {
	gorm.Model
	ID       int
	Username string
	Created  time.Time
}

type Item struct {
	gorm.Model
	ID          int
	Location    string
	Kind        string
	Icon        string
	Title       string
	Description string
	Uploaded    time.Time
	Duration    time.Duration
	Upvotes     int
	Downvotes   int
	Owner       User `gorm:"embedded"`
}

func (i *Item) Date() string {
	return i.CreatedAt.Format("15:04, Mon Jan _2 2006")
}

var db *gorm.DB

func OpenDB(dbloc string) (err error) {
	db, err = gorm.Open(sqlite.Open(dbloc), &gorm.Config{})
	if err != nil {
		return
	}

	// seed randy numbers
	rand.Seed(time.Now().UnixNano())

	// migrate moment
	err = db.AutoMigrate(&Item{}, &User{})
	return err
}

func GetItem(id int) (*Item, error) {
	var item Item
	// grab matching item from the database
	err := db.First(&item, "id = ?", id).Error
	return &item, err
}

func NewItem(
	title string, kind string, icon string, desc string, owner User, loc string) (int, error) {

	id := rand.Int()

	i := &Item{
		ID:          id,
		Title:       title,
		Kind:        kind,
		Icon:        icon,
		Description: desc,
		Owner:       owner,
		Location:    loc,
		Upvotes:     0,
		Downvotes:   0,
	}

	err := db.Create(i).Error

	if err != nil {
		return 0, err
	}

	return id, nil
}

func Latest() ([]Item, error) {
	var items []Item
	q := db.Table("items").Order("created_at").Limit(20).Find(&items)
	return items, q.Error
}